﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace test_console
{

    /// <summary>A very simple class. Contains only some internal variables.</summary>
    public class SimpleClass
    {

        public int A, B;

        protected int C;

        public double X, Y;

        /// <summary>Prints internal state of the object.</summary>
        public virtual void Print()  // note the virtual keyword; must be used if we want to override the method in derived classes
        {
            Console.WriteLine();
            Console.WriteLine("State of SimpleClass object: ");
            Console.WriteLine("  A = " + A.ToString());
            Console.WriteLine("  B = " + B.ToString());
            Console.WriteLine("  C = " + C.ToString());
            Console.WriteLine("  X = " + X.ToString());
            Console.WriteLine("  Y = " + Y.ToString());
            Console.WriteLine();
        }

    }

    /// <summary>Class that inherits form <see cref="SimpleClass"/>.</summary>
    public class DerivedClass : SimpleClass
    {
        public void SetC(int c)
        {
            this.C = c;
        }

        /// <summary>Returns sum of integer internal variables.</summary>
        /// <returns></returns>
        public int Sum()
        {
            return A + B + B;
        }

        public void PrintSum()
        {
            int sum = this.Sum();
            Console.WriteLine();
            Console.WriteLine("  A+B+C = " +sum.ToString());
            Console.WriteLine();
        }

        /// <summary>The Print method of <see cref="DerivedClass"/>. Overridden from base class.</summary>
        public override void Print()
        {
            base.Print(); // first, we call the base class' Print method
            PrintSum();  // then, call PrintSum() in addition.
        }

    }



    public class Person
    {

        private Person()
        {
            ++LastId;
            Id = LastId;
        }

        public Person(string name, string surName)
            : this()
        {
            this.Name = name;
            this.SurName = surName;
        }

        public Person(string name, string surName, Date birthDate): this(name, surName)
        {
            DateOfBirth = birthDate;  // keyworkd this can be omitted if there is no ambiguity.
        }

        public string Name;

        public string SurName;

        public Date DateOfBirth;

        private static int LastId = 0;

        public int Id;

        /// <summary>Returns a stirng that introduces the current object.</summary>
        public virtual string GetIntroductionString()
        {
            return ("Person: " + Environment.NewLine);
        }

        /// <summary>Returns string representation of the current object.</summary>
        public override string ToString()
        {
            string birthDateString = "Not specified";
            if (DateOfBirth != null)
                birthDateString = DateOfBirth.ToString();
            string ageString = "";
            if (DateOfBirth!=null)
                ageString = "  Age: " + DateOfBirth.AgeString() + Environment.NewLine;
            return GetIntroductionString()
                + "  Name: " + Name.ToString() + Environment.NewLine
                + "  Surname: " + SurName.ToString() + Environment.NewLine
                + "  DateofBirth: " + birthDateString + Environment.NewLine
                + ageString;
        }

    }



    /// <summary>Represents an employee. This class inherits from class <see cref="Person"/>.</summary>
    public class Employee : Person
    {

        public Employee(string name, string surName)
            : base(name, surName)
        {  }

        public Employee(string name, string surName, Date dateOfBirth)
            : base(name, surName, dateOfBirth)
        {  }

        public Employee(string name, string surName, Date dateOfBirth, CompanyClass company)
            : base(name, surName, dateOfBirth)
        {
            this.Company = company;
        }

        public Employee(Person person, CompanyClass company): this(person.Name, person.SurName, person.DateOfBirth, company)
        {  }

        /// <summary>Company where a person is employed. This is additional member of <see cref="Employee"/>
        /// with respect to <see cref="Person"/>.</summary>
        public CompanyClass Company;

                
        /// <summary>Returns a stirng that introduces the current object; 
        /// overridden from base class.</summary>
        public override string GetIntroductionString()
        {
            return ("Employee: " + Environment.NewLine);
        }

        public override string ToString()
        {
            string companyString = "Not specified." + Environment.NewLine;
            if (Company != null)
                companyString = Environment.NewLine + Company.ToString();
            return base.ToString() + "  Employee's Company: " + companyString;
        }
    }


    /// <summary>Represents a company.</summary>
    public class CompanyClass
    {

        private CompanyClass()
        {
            ++LastId;
            Id = LastId;
        }

        public CompanyClass(string name)
            : this()
        {
            Name = name;
        }

        private static int LastId = 0;

        public int Id;

        public string Name;

        public override string ToString()
        {
            return "  Company ID: " + Id.ToString() + Environment.NewLine
                + "  Company Name: " + Name + Environment.NewLine;
        }
    }


    /// <summary>Class for storing a date.</summary>
    public class Date
    {

        public Date(int year, int month, int day)
        {
            this.Year = year;
            this.Month = month;
            this.Day = day;
        }

        public int Year;

        public int Month;

        public int Day;

        /// <summary>Creates a date that represents today's day.</summary>
        public static Date Today()
        {
            DateTime d = DateTime.Now; // get current date by system utilities
            return new Date(d.Year, d.Month, d.Day);
        }

        /// <summary>Creates a date that represents a date in the past, which was specified time before today.</summary>
        /// <param name="yearsBefore">How many years before today was that date.</param>
        /// <param name="monthsBefore">How many months before today was that date.</param>
        /// <param name="daysBefore">How many days before today was that date.</param>
        /// <returns></returns>
        public static Date TimeBefore(int yearsBefore, int monthsBefore, int daysBefore)
        {
            DateTime d = DateTime.Now;
            d = d.AddYears(-yearsBefore);
            d = d.AddMonths(-monthsBefore);
            d = d.AddDays(-daysBefore);
            return new Date(d.Year, d.Month, d.Day);
        }

        public override string ToString()
        {
            return Day.ToString() + ". " + Month.ToString() + ". " + Year.ToString();
        }

        /// <summary>Returns a string represating age of somebody that was born on the current date.</summary>
        /// <returns></returns>
        public string AgeString()
        {
            DateTime dateOfBirth = new DateTime(Year, Month, Day);
            DateTime currentDate = DateTime.Now;
            int ageInYears = 0;
            int ageInMonths = 0;
            int ageInDays = 0;
            ageInDays = currentDate.Day - dateOfBirth.Day;
            ageInMonths = currentDate.Month - dateOfBirth.Month;
            ageInYears = currentDate.Year - dateOfBirth.Year;
            if (ageInDays < 0)
            {
                ageInDays += DateTime.DaysInMonth(currentDate.Year, currentDate.Month);
                ageInMonths = ageInMonths--;

                if (ageInMonths < 0)
                {
                    ageInMonths += 12;
                    ageInYears--;
                }
            }
            if (ageInMonths < 0)
            {
                ageInMonths += 12;
                ageInYears--;
            }
            return ageInYears.ToString() + " years, " + 
                ageInMonths.ToString() + " months, " + ageInDays.ToString() + " days";
        }

    }

}
