﻿#region About
// Course material: C# for numerical methods
// Copyright (c) 2009-2010, Igor Grešovnik
#endregion About


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NumLib;
using NumLib.Classes;


// Preprosta konzolna aplikacija, ki prebere parametre in nekaj izračuna:

namespace test_console
{
    class Program
    {


        static void Main(string[] args)
        {

            //Vector.Example();
            
            //// A simple console application to learn the basics:
            // TestConsoleApplication();

            //// Simple classes tests:
            // TestClassesSimple();
            // TestClasses();

            //// Test of formatting of numbers for output methods:
            // TestFormatting();

            //// Example use of the Complex class:
            //Complex.Example();  // Example with complec numbers

            //// Example use of the ComplexVector Class:
            //ComplexVector.Example();

            //// Matrix example:
            // Matrix.Example();

            //// Example use of the Matrix class:
            // SparseMatrix.Example();

            //// Solution of nonlinear equations:
            //EquationSolving.ExampleStatic();
            EquationSolving.ExampleObjectOriented();

        }

        static void TestConsoleApplication()
        {
            try
            {
                int n;
                double x, val;
                string str;
                Console.WriteLine();
                Console.Write("Argument funkcije: ");
                str = Console.ReadLine();
                x = double.Parse(str);
                Console.Write("potenca: ");
                str = Console.ReadLine();
                n = int.Parse(str);
                Console.WriteLine();
                // Izračun vrednosti nekaterih funkcij in izpis rezultatov:
                val = Pow.Square(x);
                Console.WriteLine("Kvadrat " + x.ToString() + ": " + val.ToString());
                Console.WriteLine();
                val = Pow.Cubic(x);
                Console.WriteLine("Kub " + x.ToString() + ": " + val.ToString());
                Console.WriteLine();
                val = Pow.Power(x,n);
                Console.WriteLine("Potenca " + x.ToString() + " na " + n.ToString() + ": " + val.ToString());
                // Console.WriteLine("Potenca " + x + " na " + n + ": " + val);
            }
            catch (Exception ex)
            {
                // Tu lovimo napake. Poskusi za argument funkcije vstaviti neštevilčno vrednost,
                // da se bo izvedel ta blok!
                Console.WriteLine();
                Console.WriteLine("Prišlo je do napake. Opis: " + ex.Message);
                Console.WriteLine();
            }
            finally
            {
                // Ta blok se izvede v vsakem primeru, tudi, če pride do napake.
                // Izvajanje programa se v primeru napake prekine in se nadaljuje v prvem
                // catch (...) {...} bloku za mestom napake.
                Console.WriteLine();
                Console.WriteLine("Izvajanje programa je končano.");
            }
        } // TestConsoleApplication()


        /// <summary>Test of simple classes.</summary>
        static void TestClassesSimple()
        {
            // Definition and creation of objects:
            Console.WriteLine("Basic classes: ");
            SimpleClass a, b;
            a = new SimpleClass();
            a.A = 1;
            a.B = 2;
            a.X = 1.1;
            a.Y = 1.2;
            // a.C = 3; // this is illegal because member C is not public
            b = new SimpleClass();
            b.A = 10;
            b.B = 20;
            b.X = 10.1;
            b.Y = 10.2;
            Console.WriteLine();
            Console.WriteLine("Object a: ");
            a.Print();
            Console.WriteLine("Object b: ");
            b.Print();
            Console.WriteLine();
            // Reference assignment: 
            b = a;
            Console.WriteLine("Object b after assignment of reference: ");
            b.Print();
            a.A = 9999;
            Console.WriteLine();
            Console.WriteLine("Object b after change of object a): ");
            b.Print();
        }


        /// <summary>Test of formatting numerical output.</summary>
        /// <remarks>
        /// <para>See also:</para>
        /// <para>http://msdn.microsoft.com/en-us/library/dwhawy9k.aspx</para>
        /// <para>http://msdn.microsoft.com/en-us/library/b1csw23d.aspx</para>
        /// </remarks>
        public static void TestFormatting()
        {
            Console.WriteLine();
            Console.WriteLine("Formatting numbers: ");
            Console.WriteLine();
            double a = Math.PI / 2.0;
            double b = 2.3;
            double a0 = a / 1.0e6;
            double a1 = a * 1e6;
            double b0 = b / 1.0e6;
            double b1 = b * 1.0e6;
            Console.WriteLine("Different ways of output formatted number: ");
            Console.WriteLine("By Console.Writeline(), unformatted (using string concatanation object.ToString() implicitly): ");
            Console.WriteLine("  a = " + a + ", a0 = " + a0 + ", a1 = " + a1);
            Console.WriteLine("By Console.Writeline() with argument references in curly brackets - unformatted: ");
            Console.WriteLine("  a = {0}, a0 ={1}, a1 = {2}", a, a0, a1);
            Console.WriteLine("By Console.Writeline() with argument references in curly brackets - formatted: ");
            Console.WriteLine("  a = {0,-15:G3}, a0 ={1,15:0.0000E0}, a1 = {2,15:0.0000E0}", a, a0, a1);
            Console.WriteLine("By Console.Writeline() and object.ToString() with format specifier: ");
            Console.WriteLine("  a = " + a.ToString("G4") + ", a0 = " + a0.ToString("G4") + ", a1 = " + a1.ToString("G4"));
            Console.WriteLine("By String.Format(): ");
            Console.WriteLine(String.Format("  a = {0,-15:0.0000E0}, a0 ={1,15:0.0000E0}, a1 = {2,15:0.0000E0}", a, a0, a1));
            Console.WriteLine();
            Console.WriteLine("Demonstration of formatting on different numbers: ");
            Console.WriteLine("Formatting with E4 (exponential, 4 digits), occupying 8 places:");
            Console.WriteLine("  a = {0,8:E4}, a0 ={1,8:E4}, a1 = {2,8:E4}", a, a0, a1);
            Console.WriteLine("  b = {0,8:E4}, b0 ={1,8:E4}, b1 = {2,8:E4}", b, b0, b1);
            Console.WriteLine("Formatting with F4 (fixed point, 4 digits), occupying 8 places:");
            Console.WriteLine("  a = {0,8:F4}, a0 ={1,8:F4}, a1 = {2,8:F4}", a, a0, a1);
            Console.WriteLine("  b = {0,8:F4}, b0 ={1,8:F4}, b1 = {2,8:F4}", b, b0, b1);
            Console.WriteLine("Formatting with G4 (general, 4 digits), occupying 8 places:");
            Console.WriteLine("Remark: general is the most compact of either fixed point or exponential notation");
            Console.WriteLine("  a = {0,8:G4}, a0 ={1,8:G4}, a1 = {2,8:G4}", a, a0, a1);
            Console.WriteLine("  b = {0,8:G4}, b0 ={1,8:G4}, b1 = {2,8:G4}", b, b0, b1);
        }


        /// <summary>Test of classes.</summary>
        public static void TestClasses()
        {
            // More complex class usage
            Console.WriteLine();
            Console.WriteLine("Usage of Person and Employee class: ");
            Console.WriteLine();
            Person
                p1 = new Person("Janez", "Cehner"),
                p2 = new Person("Marjana", "Tekavec", new Date(1969, 7, 22));
            Console.WriteLine();
            Console.WriteLine("Person a: " + Environment.NewLine + p1.ToString());
            Console.WriteLine("Person b: " + Environment.NewLine + p2.ToString());
            // Assign date of birth to person a:
            p1.DateOfBirth = Date.TimeBefore(29, 04, 14);
            Console.WriteLine("Person a after assigning date of birth: ");
            Console.WriteLine(p1.ToString());
            // Create an object holding data of a company:
            CompanyClass strojniskaFirma = new CompanyClass("Iskra avtoelektrika");
            CompanyClass letalskaFirma =  new CompanyClass("Adria airways");
            Employee emp1 = new Employee(p1, strojniskaFirma);
            Employee emp2 = new Employee(p2, letalskaFirma);
            Console.WriteLine();
            Console.WriteLine("Employees: ");
            Console.WriteLine("Employee 1: " + Environment.NewLine + emp1.ToString());
            Console.WriteLine("Employee 2: " + Environment.NewLine + emp2.ToString());
        }

    }
}
